@@ -23,6 +23,7 @@ import zc.lockfile
 import socket
 import tempfile
 import urlparse
+import time
 from pkg_resources import parse_version
 from BeautifulSoup import BeautifulSoup
 from glob import fnmatch
@@ -81,14 +82,14 @@ class Stats:
         return ret
 
 
-class PypiPackageList:
+class PypiPackageList(object):
     """
         This fetches and represents a package list
     """
     def __init__(self, pypi_xmlrpc_url='http://pypi.python.org/pypi'):
         self._pypi_xmlrpc_url = pypi_xmlrpc_url
 
-    def list(self, filter_by=None):
+    def list(self, filter_by=None, incremental=False, fetch_since_days=7):
         server = xmlrpclib.Server(self._pypi_xmlrpc_url)
         packages = server.list_packages()
         if not filter_by:
@@ -99,7 +100,14 @@ class PypiPackageList:
             if not True in [fnmatch.fnmatch(package, f) for f in filter_by]:
                 continue
             filtered_packages.append(package)
-        return filtered_packages
+
+        if incremental:
+            changelog = server.changelog(int(time.time() - fetch_since_days*24*3600))
+            changed_packages = [tp[0] for tp in changelog]
+            changed_packages = [package for package in changed_packages if package in filtered_packages]
+            return changed_packages
+        else:
+            return filtered_packages
 
 
 class PackageError(Exception):
@@ -460,6 +468,7 @@ class Mirror:
             self.cleanup(package_list, verbose)
         if create_indexes:
             self.index_html()
+            full_list.sort()
             self.full_html(full_list)
 
         for line in stats.getStats():
@@ -641,6 +650,10 @@ def run(args=None):
                       default=False, help='verbose on')
     parser.add_option('-f', '--log-filename', dest='log_filename', action='store',
                       default=False, help='Name of logfile')
+    parser.add_option('-I', '--initial-fetch', dest='initial_fetch', action='store_true',
+                      default=False, help='Initial PyPI mirror fetch')
+    parser.add_option('-U', '--update-fetch', dest='update_fetch', action='store_true',
+                      default=False, help='Perform incremental update of the mirror')
     parser.add_option('-c', '--log-console', dest='log_console', action='store_true',
                       default=False, help='Also log to console')
     parser.add_option('-i', '--indexes-only', dest='indexes_only', action='store_true',
@@ -669,7 +682,13 @@ def run(args=None):
     if options.log_filename:
         log_filename = options.log_filename
 
-    package_list = PypiPackageList().list(package_matches)
+
+    if options.initial_fetch:
+        package_list = PypiPackageList().list(package_matches, incremental=False)
+    elif options.update_fetch:
+        package_list = PypiPackageList().list(package_matches, incremental=True)
+    else: 
+        raise ValueError('You must either specify the --initial-fetch or --update-fetch option ')
     mirror = Mirror(config["mirror_file_path"])
     lock = zc.lockfile.LockFile(os.path.join(config["mirror_file_path"], config["lock_file_name"]))
     LOG = getLogger(filename=log_filename,
